/************************************************
            Cornell University ECE 4760
                Spring 2009
    Electric Etch - the motorized Etch-a-Sketch

                    by
   Kirill Kalinichev and Seong-hee (Emily) Lee
           kk383             sl486
*************************************************/

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>		// delays used to meet hold times of stepper motors
#include <avr/pgmspace.h>	// character points defined in program memory
#include <stdio.h>			// used for linking UART to print functions
#include <string.h>			// used for formatting strings

#include "etch_chars.h"		// Character point arrays
#include "uart.h"

#define TRUE 	1
#define FALSE 	0

/*** Global Variables ***/

// Task counters and periods
#define HBT_PERIOD 		250		// ms, heartbeat toggle rate
#define MOTOR_PERIOD	3		// ms, fastest attainable
volatile unsigned char task_hbt, task_motor, task_tilt;
// End task counters

// Motor Definitions
#define DDRM_V		DDRA	// Vertical Motor
#define MOTOR_V		PORTA
#define CLK_V 		PA0
#define HALF_V		PA1
#define DIR_V 		PA2
#define EN_V		PA3		// number of ticks to compensate for backlash
#define BACKLASH_V	35

#define DDRM_H		DDRA	// Horizontal Motor
#define MOTOR_H		PORTA
#define CLK_H		PA4
#define HALF_H		PA5
#define DIR_H		PA6
#define EN_H		PA7
#define BACKLASH_H	25		// number of ticks to compensate for backlash

#define DDRM_T 		DDRC	// Tilt motor
#define MOTOR_T		PORTC
#define CLK_T		PC4
#define HALF_T		PC5
#define DIR_T		PC6
#define EN_T		PC7

#define ENABLE_MOTORS 	MOTOR_V |= (1<<EN_V); MOTOR_H |= (1<<EN_H)
#define DISABLE_MOTORS 	MOTOR_V &= ~(1<<EN_V); MOTOR_H &= ~(1<<EN_H)

signed char last_H_dir, last_V_dir;
// End motor definitions

// Absolute position variables
signed int desired_H_pos, desired_V_pos;	// the desired position on the EAS (absolute)
signed int H_pos, V_pos;					// the actual position (absolute)
signed int H_anchor_pos, V_anchor_pos;

#define MAX_TILT 		40	// ~3.25deg steps, number of tilt steps in one direction
#define TILT_PERIOD		50	// ms, period between tilt steps
unsigned char tilt_pos;		
// End absolute position variables

// Line-drawing state variables
char drawing_in_progress;			// in progress flag
int e;
signed int dx, dy;
signed char s1, s2, xchange;
// End line-drawing variables

// Draw state variables
char draw_state;
#define IDLE 			0		// when not drawing anything
#define RESET_REQ 		1		// for requesting cursor back to initial position
#define RESETTING		2		// for when it's resetting
#define DRAW			3		// when drawing a character
#define TILT			4		// when clearing the screen, i.e., tipping the EAS
#define TILT_THEN_DRAW	5
#define HOME			6		// when moving the cursor back to the start position

#define HOME_H	-43		// these values were found to correct for backlash well.
#define HOME_V 	-5

char point_idx;		// Used for traversing between points of a single character
int* current_char_P;	// Pointer to current character being drawn (program space)
// End draw-state variables

// Time variables
unsigned int ms_count;	// MS counter, counts up to 1000 on 1ms time base to update clock
volatile unsigned char t_sec, t_min, t_hour;	// Current second, minute, and hour
volatile unsigned int t_day = 1;				// Current day, starting from Jan 1
volatile char draw_new_time;					// flag set by interrupt to request new time
// End time variables

// USART variables
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);
#define RX_MAX_LEN 256
volatile char rx_buffer[RX_MAX_LEN];	// Recieve buffer
volatile char unsigned rx_idx;			// Recieve buffer index
volatile char rx_ready;					// Recieve ready flag

#define TX_MAX_LEN 256
char tx_buffer[TX_MAX_LEN];		// Transmit buffer
volatile char unsigned tx_idx;	// Transmit buffer index
volatile char tx_ready = 1;		// Transmit ready flag

#define RX_BUFF_OVR 	1		// Recieve buffer overflow
#define RX_UNKNOWN_CMD	2		// Unrecognized command
#define RX_WRONG_FORMAT	3		// Unrecognized format of recognized command
volatile char uart_error = 0;	// Global UART error flag
// End USART variables

// Time or message board mode variables
char mode;
#define CLOCK 	0
#define MESSAGE 1
// End mode variables

// Message mode variables
char message[256];
char message_state;
#define DONE 			0		// message done
#define WAITING			1		// when waiting for message
#define CLEAR_MSG		2		// when clearing screen and returning cursor to top for new message
#define DRAW_MSG		3		// while drawing the message
#define NEW_LINE_REQ	4		// for coming back to home before dropping down a line
#define NEW_LINE		5		// for dropping down a line
// End message mode variables
/*** End Global Variables ***/


/*** Function Prototypes ***/
void initialize(void);				// Sets up I/O, timer
void toggle_hbt(void);				// Toggles the heartbeat LED for debugging
void progress_motors(void);			// Moves motors from current point to desired point
void load_next_point(void);			// Loads desired points
int* get_char(char c);				// Returns the prog space address of char c's points
void update_tilt(void);				// Manages the tilting motor which clears the screen
void getstr(void);					// Readies the MCU to recieve a string over serial
void putstr(void);					// Initiates USART transmit of tx_buffer string
void process_rx(void);				// Processes strings recieved over serial
void set_time(void);				// Sets the MCU time
unsigned char check_message(void);	// checks the recieved message and returns number of discarded characters
void reset(char t);					// resets variables to set starting position for clock and msg modes
/*** End Function Prototypes ***/

char string[256];
unsigned char char_idx;

/*** Function Implementation ***/
int main(void) {
	initialize();		

	// Send welcome message
	while(!tx_ready);
	sprintf(tx_buffer,"Welcome to the Electric Etch!\rDefault mode is Clock, and the current time is %2i:%02i.\n\rType 'help' to see all commands.\n\r\n\r", t_hour, t_min);
	putstr();

	getstr();	// Get ready to recieve messages

	// initialize clock mode by resetting to clock home positionn
	reset('c');	// c indicates clock mode
	
	while(1) {

		// If a USART message is ready, process it.
		if(rx_ready) {
			process_rx();
		}	
	
		if(0 == task_hbt) { task_hbt = HBT_PERIOD; toggle_hbt(); }		
		if(0 == task_motor) { task_motor = MOTOR_PERIOD; progress_motors(); }
		if(0 == task_tilt) { task_tilt = TILT_PERIOD;	update_tilt(); }

		// here comes the onslaught of interlinked state machines
		// all written at very odd hours of the night. Forgive the comlete illegibility.
	
		if(mode == CLOCK) {
			if(draw_state == IDLE && draw_new_time == TRUE) {	// if idle and there's a new time waiting to be drawn
				draw_new_time = FALSE;	// clear time flag
				draw_state = TILT_THEN_DRAW;	// next time, it will tilt and continue to draw new time
				sprintf(string, "%2i:%02i", t_hour, t_min);
				char_idx = 0;
			} else if(draw_state == DRAW) {
				if(string[char_idx] != 0) {	// if letters remain to be printed
					current_char_P = get_char(string[char_idx]);	// assign next letter
					ENABLE_MOTORS;		// enable motors
					_delay_ms(5);		// wait for motor transients to settle
					// char_idx will be incremented by load_next_point() if necessary
				} else {	
					// reset back to home, the starting position
					draw_state = HOME;
					ENABLE_MOTORS;
					_delay_ms(5);
				}
			}	
		
		} else {	// otherwise, mode is message

			if(draw_state == IDLE && message_state == CLEAR_MSG) {				
				draw_state = TILT_THEN_DRAW;
				sprintf(string, "HI");
				char_idx = 0;
			} else if(draw_state == DRAW) {
				if(message[char_idx] != 0) {	// if letters remain to be printed
					
					if((char_idx+1) % 8 == 0) {	// if 7 characters have been drawn, start a new line
						message_state = NEW_LINE_REQ;
						draw_state = HOME;
						ENABLE_MOTORS;
						_delay_ms(5);
					} else {
						current_char_P = get_char(message[char_idx]);	// assign next letter
						message_state = DRAW_MSG;
						ENABLE_MOTORS;		// enable motors
						_delay_ms(5);		// wait for motor transients to settle
						// char_idx will be incremented by load_next_point() if necessary
					}
				} else {
					// reset back to home, the starting position
					draw_state = HOME;
					ENABLE_MOTORS;
					_delay_ms(5);
				}
			}
		}


		// If there's a problem with the recieved message, respond in a friendly manner
		if(uart_error) {
			while(!tx_ready);
			switch(uart_error) {
				case RX_BUFF_OVR:
					sprintf(tx_buffer, "Sorry, your command was too long. Try again!\n\r");
					break;
				case RX_UNKNOWN_CMD:
					sprintf(tx_buffer, "Sorry, but that's an unknown command. Type 'help' to see known commands.\n\r");
					break;
				case RX_WRONG_FORMAT:
					sprintf(tx_buffer, "Sorry, but that command is not formatted properly. Type 'help' to check command format.\n\r");
					break;
				default:
					sprintf(tx_buffer, "Unknown error (code: %i)!\n\r", uart_error);
					break;
			}
			putstr();
			getstr();
			uart_error = 0;
		}

	}	// end while
}


void load_next_point(void) {
	// State machine that manages desired positions
	
	if(mode == CLOCK) {	// define seperate state behaviors for the two modes
	
		if(draw_state == RESET_REQ) {
			draw_state = RESETTING;	// go on and enter reset points
			MOTOR_V |= (1<<EN_V); MOTOR_H |= (1<<EN_H);	// turn on motors
		} else if(draw_state == RESETTING) {
			if(point_idx < pgm_read_word(current_char_P) - 1) {	// if still not done resetting
				point_idx++;	// increment to next reset point
			} else {	// otherwise, done resetting
				point_idx = 0;			// clear point index
				current_char_P = 0;		// clear current character so that no letters are drawn
				DISABLE_MOTORS;	// disable knob motors
				draw_state = IDLE;		// go to idle. While loop will assign chars to draw.
				// reset all absolute position parameters
				H_pos = 0; V_pos = 0;
				desired_H_pos = 0; desired_V_pos = 0;
				H_anchor_pos = 0; V_anchor_pos = 0;
			}
		} else if(draw_state == DRAW) {
			if(point_idx < pgm_read_word(current_char_P) - 1) {	// if more points remain in current character
				point_idx++;
			} else {	// otherwise, we've reached the end of the character points.
				char_idx++;		// increment character index for while(1) loop to determine if there's more chars to draw
				point_idx = 0;	// reset point index to zero, for next time
				H_anchor_pos = H_pos; V_anchor_pos = V_pos;	// set the new anchor
				return;		
			}	
		} else if(draw_state == HOME) {	// artificial method of setting back to home position of line

			if(H_pos == HOME_H && V_pos == HOME_V) {	// if back at home position
				draw_state = IDLE;	
				H_pos = 0; V_pos = 0; 
				desired_H_pos = 0; desired_V_pos = 0;
				H_anchor_pos = 0; V_anchor_pos = 0;
				DISABLE_MOTORS;
				_delay_ms(5);	
				return;
			} else {	// if not at home position yet
				desired_H_pos = HOME_H;
				desired_V_pos = HOME_V;
				return;
			}
		} else {
			return;		// for all other states
		}
	} else {	// if mode != CLOCK ... largely a repeat
		if(draw_state == RESET_REQ) {
			draw_state = RESETTING;
			ENABLE_MOTORS;
		} else if(draw_state == RESETTING) {
			if(point_idx < pgm_read_word(current_char_P) - 1) {	// if still not done resetting
				point_idx++;	// increment to next reset point
			} else {	// otherwise, done resetting
				point_idx = 0;			// clear point index
				current_char_P = 0;		// clear current character so that no letters are drawn
				DISABLE_MOTORS;	// disable knob motors
				draw_state = IDLE;		// go to idle. While loop will assign chars to draw.
				// reset all absolute position parameters
				H_pos = 0; V_pos = 0;
				desired_H_pos = 0; desired_V_pos = 0;
				H_anchor_pos = 0; V_anchor_pos = 0;
			}
		} else if(draw_state == DRAW) {
			if(point_idx < pgm_read_word(current_char_P) - 1) {	// if more points remain in current character
				point_idx++;
			} else {	// otherwise, we've reached the end of the character points.
				char_idx++;		// increment character index for while(1) loop to determine if there's more chars to draw
				point_idx = 0;	// reset point index to zero, for next time
				H_anchor_pos = H_pos; V_anchor_pos = V_pos;	// set the new anchor
				return;		
			}	
		} else if(draw_state == HOME) {	// hackish method of setting back to home position of line
			
			if(message_state == NEW_LINE_REQ) {
				if(H_pos == HOME_H && V_pos == HOME_V) {	// reached home of current line
					desired_H_pos = HOME_H;
					desired_V_pos = HOME_V - 150;	// start dropping down a line
					message_state = NEW_LINE;
					return;
				} else {	// if not yet reached, continue to go to home of current line
					desired_H_pos = HOME_H;
					desired_V_pos = HOME_V;
					return;
				}

			} else if(message_state == NEW_LINE) {	// if dropping down
				if(H_pos == HOME_H && V_pos == HOME_V - 300) {	// reached new line home
					char_idx++;
					point_idx = 0;	
					current_char_P = get_char(message[char_idx]);	
					draw_state = DRAW;
					H_pos = 0; V_pos = 0; // absolute positions reset
					desired_H_pos = 0; desired_V_pos = 0;
					H_anchor_pos = 0; V_anchor_pos = 0;
					//DISABLE_MOTORS;
					//_delay_ms(5);	
					return; //fall through
				} else {		// if not yet reached new line, keep dropping down
					desired_H_pos = HOME_H;
					desired_V_pos = HOME_V - 300;
					return;
				}
			} else {	// normal home mode, for single-line messages
				if(H_pos == HOME_H && V_pos == HOME_V) {
					draw_state = IDLE;	
					H_pos = 0; V_pos = 0; 
					desired_H_pos = 0; desired_V_pos = 0;
					H_anchor_pos = 0; V_anchor_pos = 0;
					DISABLE_MOTORS;
					_delay_ms(5);	
					return;
				} else {
					desired_H_pos = HOME_H;
					desired_V_pos = HOME_V;
					return;
				}
			}
		
		} else {
			return;		// for all other states
		}
	}

	// funny thing about GCC: it sees that c is a pointer
	// to 16-bit words, not 8-bit bytes. So when adding an 
	// offset to the pointer as done below, it automagically
	// treats the offset as a WORD index, i.e., c + 2 is the
	// third WORD in the array, not the third byte.

	// Recall that each char's first index is the number of 
	// {x,y} int PAIRS in the remainder of the array.
	
	if(current_char_P == 0) return;		
	
	int rel_H_pos = pgm_read_word(current_char_P + (point_idx<<1) + 1);
	int rel_V_pos = pgm_read_word(current_char_P + (point_idx<<1) + 2);



	int new_desired_H_pos = H_anchor_pos + (rel_H_pos);
	int new_desired_V_pos = V_anchor_pos + (rel_V_pos);



	// get new direction
	signed char new_H_dir, new_V_dir;

	if(H_pos < new_desired_H_pos) new_H_dir = 1;
	else if(H_pos > new_desired_H_pos) new_H_dir = -1;
	else new_H_dir = last_H_dir;

	if(V_pos < new_desired_V_pos) new_V_dir = 1;
	else if(V_pos > new_desired_V_pos) new_V_dir = -1;
	else new_V_dir = last_V_dir;


	// The next two if statements manage the backlash operation.
	// The Etch-a-sketch suffers from significant backlash. 	
	// If any of the servos change direction, they apply a set
	// number of ticks in the new direction to "start it going".
	
	if(last_H_dir != new_H_dir) {	// backlash correction for horizontal
		
		char temp_H = MOTOR_H;

		if(new_H_dir == 1) MOTOR_H |= (1<<DIR_H);
		else MOTOR_H &= ~(1<<DIR_H);

		char i;
		for(i=0; i<BACKLASH_H; i++) {
			MOTOR_H |= (1<<CLK_H);
			_delay_us(5);
			MOTOR_H &= ~(1<<CLK_H);
			_delay_ms(3);
		}
		MOTOR_H = temp_H;
	}
	
	if(last_V_dir != new_V_dir) {	// backlash correction for vertical
	
		char temp_V = MOTOR_V;
		if(new_V_dir == 1) MOTOR_V |= (1<<DIR_V);
		else MOTOR_V &= ~(1<<DIR_V);

		char i;
		for(i=0; i<BACKLASH_V; i++) {
			MOTOR_V |= (1<<CLK_V);
			_delay_us(5);
			MOTOR_V &= ~(1<<CLK_V);
			_delay_ms(3);
		}
		MOTOR_V = temp_V;
	}	

	desired_H_pos = new_desired_H_pos;
	desired_V_pos = new_desired_V_pos;

}


void process_rx(void) {
	// Processes a recieved command
	// strcmp returns 0 if strings match exactly
	if(message_state == WAITING) {	// if waiting for a message after recieving 'message' command
		if(check_message()) { 	// if message is okay
		
			message_state = CLEAR_MSG;
		} else {
			while(!tx_ready);
			sprintf(tx_buffer, "Sorry, that message was too long. Maximum length is 21 characters.\n\r");
			putstr();
			message_state = DONE;
		}
	} else if(strcmp(rx_buffer, "help") == 0) {				// type out help commands
		while(!tx_ready);
		sprintf(tx_buffer, "Here's a list of possible commands:\n\r\t'help' - shows this message\n\r");
		putstr();
		while(!tx_ready);
		sprintf(tx_buffer, "\t'set time hh:mm' - sets the Etch time to hh:mm, where 0<=hh<=23 and 0<=mm<=59.\n\r");
		putstr();
		while(!tx_ready);
		sprintf(tx_buffer, "\t'message' - sets the Etch to recieve a message of up to 21 characters. The next command sent will be considered the message. Send this command to set a new message.\n\r");
		putstr();
		while(!tx_ready);
		sprintf(tx_buffer, "\t'clock' - sets the Etch to clock mode. Draws the time every minute.\n\r");
		putstr();
	} else if(strncmp(rx_buffer, "set time",8) == 0) {	// set time
		set_time();
	} else if(strcmp(rx_buffer, "message") == 0) {		// set to message mode
		while(!tx_ready);
		sprintf(tx_buffer, "Please enter a new message of up to 21 characters.\n\r");
		putstr();
		reset('m');			// sets mode == message
		message_state = WAITING;
	} else if(strcmp(rx_buffer, "clock") == 0) {		// set to clock mode
		while(!tx_ready);
		if(mode != CLOCK) {
			sprintf(tx_buffer, "Setting to clock mode. Hold on while screen clears.\n\r");
			reset('c');
		} else {
			sprintf(tx_buffer, "Already in clock mode!\n\r");
		}
		putstr();
	} else {
		uart_error = RX_UNKNOWN_CMD;	// if unknown command
	}

	getstr();
}

void reset(char t) {
	// requests a reset of cursor depending on clock or message mode
	if(t == 'c') {				
		current_char_P = (int*)reset_clock_P;
		draw_new_time = TRUE;
		mode = CLOCK;
	} else if(t == 'm') {
		current_char_P = (int*)reset_msg_P;
		mode = MESSAGE;
	} else {
		return;
	}

	H_pos = 0; V_pos = 0;
	desired_H_pos = 0; desired_V_pos = 0;
	H_anchor_pos = 0; V_anchor_pos = 0;
	point_idx = 0;
	char_idx = 0;
	draw_state = RESET_REQ;			// asserted by load_next_point();
	
}

unsigned char check_message(void) {
	// checks the recieved message, copies recognized characters into
	// message buffer, and returns the number of discarded chars. 
	// Returns whehether or not 

	if(rx_idx > 21) return 0;

	char discarded = 0;
	char temp;

	// warning! here be hacks!
	// The way the new line code is implemented ignores the 8th and 16th characters...
	// So in order to still have it work, it jumps the 8th and 16th characters with spaces.

	unsigned char read_idx=0, write_idx=0;

	for(read_idx=0; read_idx < rx_idx; read_idx++) {
		temp = rx_buffer[read_idx];
		if(temp >= 97 && temp <= 122) temp -= 32;	// to lowercase
		if((int*)pgm_read_word(&(ascii_table[(unsigned)temp])) == 0) {	
			// if character map does not exist, fill in a space
			temp = ' ';
			discarded++;
		}		

				
		if(read_idx == 7 || read_idx == 14) {	// splice in a space for the new line hack
			message[write_idx] = ' ';
			write_idx++;		
		}		
		message[write_idx] = temp;
		write_idx++;
	}
	message[write_idx] = 0;	// null-terminate

	while(!tx_ready);
	sprintf(tx_buffer, "Drawing message with %i characters (%i unknown characters replaced with spaces):\n\r", rx_idx, discarded);
	putstr();

	while(!tx_ready);
	unsigned char i = 0;
	
	while(message[i] != 0 ){
		if(i==7 || i == 15) {
			fprintf(stdout, "\n");
		} else {
			fprintf(stdout, "%c", message[i]);
		}
		i++;
	}
	
	fprintf(stdout, "\n");

	return 1;

}

void set_time() {
	// Parses an RX set time string and sets time appropriately
	unsigned int min, hour;	// temporary matching variables
	unsigned char matched = sscanf(rx_buffer, "set time %i:%i", &hour, &min);
	if(matched == 2) {	// If sscanf matched all four integers
		t_hour = (hour > 23) ? 23 : (unsigned char) hour;
		t_min = (min > 59) ? 59 : (unsigned char) min;
		while(!tx_ready);
		sprintf(tx_buffer, "Set time to: %i:%02i. Will update screen within a minute.\n\r", t_hour, t_min);
		putstr();
	} else {
		uart_error = RX_WRONG_FORMAT;
	}
}

void update_tilt(void) {
	if(draw_state != TILT && draw_state != TILT_THEN_DRAW) return;
	
	if(! (MOTOR_T & (1<<EN_T) )){	// if not enabled, enable
		MOTOR_T |= (1<<EN_T);
		_delay_ms(5);			// wait for transients to settle
	}
	
	if(tilt_pos < MAX_TILT) { 	// tilt forward for first half of count
		tilt_pos++;
		MOTOR_T |= (1<<DIR_T);		
	} else if(tilt_pos < (MAX_TILT << 1)) {	// tilt backward for second half
		tilt_pos++;
		MOTOR_T &= ~(1<<DIR_T);
	} else {					// otherwise, finish tilting
		tilt_pos = 0;
		char i;
		for(i=0; i<50; i++) 	// wait for half a second, holding the motor in place
			_delay_ms(10);		// so that the etch stops tilting due to momentum
		MOTOR_T &= ~(1<<EN_T);	// disable motor
		draw_state = (draw_state == TILT) ? IDLE : DRAW;
		if(mode == MESSAGE) message_state = DRAW_MSG;
		return;
	}

	MOTOR_T |= (1<<CLK_T);
	_delay_us(5);
	MOTOR_T &= ~(1<<CLK_T);

}


void progress_motors(void) {
	// Progresses motors according to line-drawing algoritm

	if(drawing_in_progress == TRUE) {	// skips this the first time
		// check to see if done, and load next point if so
		if(H_pos == desired_H_pos && V_pos == desired_V_pos) {
			drawing_in_progress = FALSE;
			load_next_point();
			return;
		}
	} else {	// new line
		if(desired_H_pos < H_pos) {
			dx = H_pos - desired_H_pos;
			s1 = -1;
		} else if(desired_H_pos == H_pos) {
			dx = 0;
			s1 = 0;
		} else {
			dx = desired_H_pos - H_pos;
			s1 = 1;
		}
		if(desired_V_pos < V_pos) {
			dy = V_pos - desired_V_pos;
			s2 = -1;
		} else if(desired_V_pos == V_pos) {
			dy = 0;
			s2 = 0;
		} else {
			dy = desired_V_pos - V_pos;
			s2 = 1;
		}			
		xchange = 0;
		if(dy > dx) {
			signed temp = dx;
			dx = dy;
			dy = temp;
			xchange = 1;
		}
		e = ((int)dy<<1) - dx;
	}

	if(s1==0 && s2==0) {	// enters this the first time
		drawing_in_progress = FALSE;
		load_next_point();
		return;	
	} else {
		drawing_in_progress = TRUE;
	}

	// Record direction for backlash if nonzero
	if(s1 != 0) last_H_dir = s1;
	if(s2 != 0) last_V_dir = s2;
	
	// deterime directions and raise clocks of necessary motors
	if(e>0) {
		if(xchange==1) {	// change H (x)
			if(s1 == -1) {	// step backwards
				MOTOR_H &= ~(1<<DIR_H);
				MOTOR_H |= (1<<CLK_H);
				H_pos--;
			} else if(s1 == 1) {	// step forwards
				MOTOR_H |= (1<<DIR_H);
				MOTOR_H |= (1<<CLK_H);
				H_pos++;
			} 	// otherwise, do nothing
		} else {	// change V (y)
			if(s2 == -1) {	// step backwards
				MOTOR_V &= ~(1<<DIR_V);
				MOTOR_V |= (1<<CLK_V);
				V_pos--;
			} else if(s2 == 1) {	// step forwards
				MOTOR_V |= (1<<DIR_V);
				MOTOR_V |= (1<<CLK_V);
				V_pos++;
			}		
		}
		e = e - ((int)dx<<1);
	}
	
	if(xchange==1) {	// change V (y)
		if(s2 == -1) {	// step backwards
			MOTOR_V &= ~(1<<DIR_V);
			MOTOR_V |= (1<<CLK_V);
			V_pos--;
		} else if(s2 == 1) {	// step forwards
			MOTOR_V |= (1<<DIR_V);
			MOTOR_V |= (1<<CLK_V);
			V_pos++;
		}			
	} else {	// change H (x)
		if(s1 == -1) {	// step backwards
			MOTOR_H &= ~(1<<DIR_H);
			MOTOR_H |= (1<<CLK_H);
			H_pos--;
		} else if(s1 == 1) {	// step forwards
			MOTOR_H |= (1<<DIR_H);
			MOTOR_H |= (1<<CLK_H);
			H_pos++;
		} 	// otherwise, do nothing
	}		

	e = e + ((int)dy<<1);

    _delay_us(5);   // stepper driver's hold time
    MOTOR_V &= ~(1<<CLK_V);
    MOTOR_H &= ~(1<<CLK_H);
}


void initialize(void) {

	// Heartbeat LED output
	DDRD |= (1<<PD2);		// LED connected to port D pin 2

	// Set motor 1 output
	DDRM_V |= (1<<CLK_V) | (1<<DIR_V) | (1<<HALF_V) | (1<<EN_V);
	MOTOR_V &= ~( (1<<HALF_V) | (1<<DIR_V) );

	DDRM_H |= (1<<CLK_H) | (1<<DIR_H) | (1<<HALF_H) | (1<<EN_H);
	MOTOR_H &= ~( (1<<HALF_H) | (1<<DIR_H) );

	DDRM_T |= (1<<CLK_T) | (1<<DIR_T) | (1<<HALF_T) | (1<<EN_T);
	MOTOR_T &= ~( (1<<HALF_T) | (1<<DIR_T) );

	// If enable line control is NOT turned on, permanently enable motors
	#ifndef ENABLE_CONTROL
	MOTOR_V	|= (1<<EN_V);
	MOTOR_H |= (1<<EN_H);
	#endif

	// Set up Timer/Counter0 for 1ms time base
	TCCR0A = (1 << WGM01); 	// CTC mode
	TCCR0B = (1 << CS00) | (1 << CS01);	// 1/64th prescaler
	OCR0A = 249;			// Postscaler: 16mhz/64/(1+249) = 1000 Hz
	TIMSK0 = (1 << OCIE0A);	// Enable output compare A interrupt;

	// UART for debugging
	uart_init();	// from uart.c
	stdout = stdin = stderr = &uart_str;

	// Enable global interrupts
	sei();

}

void toggle_hbt(void) {
	// toggle heartbeat LED
	PORTD ^= _BV(PD2);
}

void putstr(void) {
	// starts transmitting a message
	tx_ready = 0;
	tx_idx = 0;
	if(tx_buffer[tx_idx] > 0) {
		UDR0 = tx_buffer[tx_idx];
		//putchar(tx_buffer[tx_idx]);
		UCSR0B |= (1<<UDRIE0);
	}
}

void getstr(void) {
	// Makes the UART RX ready to recieve
	rx_ready = 0;
	rx_idx = 0;
	UCSR0B |= (1<<RXCIE0);	// Enable rx interrupt
}

int* get_char(char c) {
	// returns the program-space address of the
	// character C. If undefined, returns the address of 
	// the space character

	int* addr = (int*)pgm_read_word(&(ascii_table[(unsigned)c]));
	
	if(addr != 0) return addr;
	else return	(int*) c__space_P;	
	
}

ISR(USART0_RX_vect) {
	// Recieved byte on USART interrupt
	char rx_char = UDR0;
	if(rx_idx == RX_MAX_LEN-1) {
		uart_error = RX_BUFF_OVR;	
		rx_idx = 0;	
		return;
	}

	if(rx_char != '\r') {	// If not a carriage return
		rx_buffer[rx_idx] = rx_char;
		rx_idx++;
	} else {
		rx_buffer[rx_idx] = 0x00;	// Zero-terminated string;
		rx_ready = 1;
		UCSR0B &= ~(1<<RXCIE0);	// Disable this interrupt	
	}		
}

ISR(USART0_UDRE_vect) {
	// Transmit buffer empty interrupt
	char tx_char = tx_buffer[++tx_idx];
	if(tx_char == 0) {
		UCSR0B &= ~(1<<UDRIE0);	// disable this interrupt
		tx_ready = 1;
	} else {
		UDR0 = tx_char;
	}
}

ISR(TIMER0_COMPA_vect) {
	if(task_hbt > 0) task_hbt--;
	if(task_motor > 0) task_motor--;
	if(task_tilt > 0) task_tilt--;	
	
	// Update D:H:M:S time
	ms_count++;
	if(ms_count > 999) {
		ms_count = 0;
		if(t_sec < 59) { t_sec++; }
		else {
			t_sec = 0;
			draw_new_time = TRUE;
			if(t_min < 59) { t_min++; }
			else {
				t_min = 0;
				if(t_hour < 23) { t_hour++; }
				else {
					t_hour = 0;
					t_day++;
				}
			}
		}
	}
}
